﻿using System;
using System.Linq;
using Microsoft.AspNetCore.Identity;
using VCommon.VApplication.Authorization;
using VCommon.VApplication.Dto;
using VCommon.VApplication.Linq.Expressions;
using VCommon.VAutoMapper;
using VCommon.VEntityFrameworkCore;
using VCommon.VOpenApi.VAspNetCore;
using VCommonCoreExample.AppServices.Basic;
using VCommonCoreExample.AppServices.Platform.Dto;
using VCommonCoreExample.Authorization;
using VCommonCoreExample.Caching;
using VCommonCoreExample.Configuration;
using VCommonCoreExample.Entity;

namespace VCommonCoreExample.AppServices.Platform
{
    [VServiceAuthorize(PermissionCodes.Platform.Tenant.EntryTenant)]
    public interface ITenantService
    {
        [VServiceAuthorize(PermissionCodes.Platform.Tenant.Query)]
        IPagedResult<TenantDto> List(TenantListRequest input);

        [VServiceAuthorize(PermissionCodes.Platform.Tenant.Create)]
        Guid Create(TenantCreateInput input);

        [VServiceAuthorize(PermissionCodes.Platform.Tenant.Update)]
        void Update(TenantUpdateInput input);

        [VServiceAuthorize(PermissionCodes.Platform.Tenant.Delete)]
        void Delete(DeleteInput input);
    }

    public class TenantService : DbTableAppServiceBase<Tenant>, ITenantService
    {
        private readonly AuthCache _authCache;

        public TenantService(AuthCache authCache)
        {
            _authCache = authCache;
        }

        public IPagedResult<TenantDto> List(TenantListRequest input)
        {
            var exp = PredicateBuilder.FilterByEnable(input)
                      & PredicateBuilder.DateTimeBetween(p => p.CreationTime, input.CreateOn)
                      & (PredicateBuilder.SearchByName(input) | PredicateBuilder.SearchBy(p => p.Code, input));

            using var repo = GetRepository(VEfDataFilters.Passive);
            var db = repo.QueryNoTracking(exp);
            var count = db.Count();
            var items = db.OrderByCreateTimeDescAndPageBy(input).ProjectionToArray<Tenant, TenantDto>();

            return new PagedResult<TenantDto>(count, items);
        }

        public Guid Create(TenantCreateInput input)
        {
            using var tRepo = GetRepository(VEfDataFilters.Passive);
            if (tRepo.CheckExist(p => p.Code == input.Code)) throw new VFriendlyException("租户代码已存在");

            //create tenant
            var tenant = new Tenant
            {
                Code = input.Code,
                Name = input.Name,
                IsEnable = input.IsEnable
            };

            tRepo.Add(tenant);
            tRepo.SaveChanges();

            //create admin user
            using var uRepo = GetRepository<User>();
            var admin = new User
            {
                TenantId = tenant.Id,
                LoginName = ProjectConst.BuildInAdminUserName,
                Name = ProjectConst.BuildInAdminUserName,
                IsStaticUser = true,
                IsEnable = true,
                Roles = "[]",
            };
            admin.Password = new PasswordHasher<User>().HashPassword(admin, input.AdminPassword);
            uRepo.Add(admin);

            uRepo.SaveChanges();

            return tenant.Id;
        }

        public void Update(TenantUpdateInput input)
        {
            using var repo = GetRepository(VEfDataFilters.Passive);
            var tenant = repo.GetEntityOrDefault(input.Id) ?? throw new VFriendlyException("找不到租户"); ;
            input.MapTo(tenant);
            repo.SaveChanges();
            _authCache.Clear(tenant.Id);
        }

        public void Delete(DeleteInput input)
        {
            using var repo = GetRepository(VEfDataFilters.Passive);
            var tenant = repo.GetEntityOrDefault(input.Id) ?? throw new VFriendlyException("找不到租户");
            tenant.Code += "_deleted_" + tenant.Id;
            repo.Delete(tenant);
            repo.SaveChanges();
            _authCache.Clear(tenant.Id);
        }
    }
}
