﻿using Microsoft.AspNetCore.Identity;
using System;
using System.Linq;
using VCommon;
using VCommon.VApplication.Authorization;
using VCommon.VApplication.Dto;
using VCommon.VApplication.Linq.Expressions;
using VCommon.VAutoMapper;
using VCommon.VEntityFrameworkCore;
using VCommon.VOpenApi.VAspNetCore;
using VCommonCoreExample.AppServices.Basic;
using VCommonCoreExample.AppServices.System.Users.Dto;
using VCommonCoreExample.Authorization;
using VCommonCoreExample.Caching;
using VCommonCoreExample.Entity;

namespace VCommonCoreExample.AppServices.System.Users
{
    [VServiceAuthorize(PermissionCodes.System.User.EntryUser)]
    public interface IUserService
    {
        [VServiceAuthorize(PermissionCodes.System.User.Query)]
        UserListOutput List(UserListRequest input);

        [VServiceAuthorize(
            PermissionCodes.System.User.Create,
            PermissionCodes.System.User.Update)]
        UserFormPrepOutput FormPrep(Guid? input);

        [VServiceAuthorize(PermissionCodes.System.User.Create)]
        Guid Create(UserCreateInput input);

        [VServiceAuthorize(PermissionCodes.System.User.Update)]
        void Update(UserUpdateInput input);

        [VServiceAuthorize(PermissionCodes.System.User.Delete)]
        void Delete(DeleteInput input);
    }

    public class UserService : DbTableAppServiceBase<User>, IUserService
    {
        private readonly AuthCache _authCache;

        public UserService(AuthCache authCache)
        {
            _authCache = authCache;
        }

        public UserListOutput List(UserListRequest input)
        {
            var exp = PredicateBuilder.FilterByEnable(input)
                  & PredicateBuilder.DateTimeBetween(p => p.CreationTime, input.CreateOn)
                  & (PredicateBuilder.SearchByName(input) | PredicateBuilder.SearchBy(p => p.LoginName, input));

            var repo = GetRepository(VEfDataFilters.Passive);
            var source = repo.QueryNoTracking(exp);
            var count = source.Count();
            var items = source.OrderByCreateTimeDescAndPageBy(input).ProjectionToArray<User, UserDto>();

            var result = new UserListOutput(count, items);

            var roleIds = result.Items.SelectMany(x => x.Roles).Distinct().ToArray();

            if (roleIds.Length > 0)
            {
                var rolRepo = repo.ForkRepository<Role>();
                var queryNoTracking = rolRepo.QueryNoTracking(x => roleIds.Contains(x.Id));
                var namedEntityDtos = queryNoTracking.Select(x => new NamedEntityDto { Id = x.Id, Name = x.Name });
                result.AvailableRoles = namedEntityDtos.ToArray();
            }
            else
            {
                result.AvailableRoles = new NamedEntityDto[0];
            }

            return result;
        }

        public UserFormPrepOutput FormPrep(Guid? input)
        {
            UserFormPrepOutput result;

            using (var ctx = GetRepository<Role>())
            {
                var items = ctx.QueryNoTracking().Select(p => new NamedEntityDto { Id = p.Id, Name = p.Name }).ToArray();
                result = new UserFormPrepOutput(items);
            }

            if (input.HasValue)
            {
                using var ctx = GetRepository(VEfDataFilters.Passive);
                result.Output = ctx.QueryProjection<UserDto>(input.Value);
            }

            return result;
        }

        public Guid Create(UserCreateInput input)
        {
            ValidRoleIds(input.Roles);
            var repo = GetRepository(VEfDataFilters.Passive);
            if (repo.CheckExist(p => p.LoginName == input.LoginName)) throw new VFriendlyException("登录名已存在");

            var entity = input.MapTo<User>();
            entity.Password = new PasswordHasher<User>().HashPassword(entity, input.Password);
            repo.Add(entity);
            repo.SaveChanges();

            return entity.Id;
        }

        private void ValidRoleIds(Guid[] roleIds)
        {
            using var ctx = GetRepository<Role>();
            var fromDb = ctx.QueryNoTracking().Select(p => p.Id).ToArray();
            if (roleIds.Except(fromDb).Any()) throw new VFriendlyException("意外的角色编号");
        }

        public void Update(UserUpdateInput input)
        {
            ValidRoleIds(input.Roles);

            var repo = GetRepository(VEfDataFilters.Passive);

            if (repo.CheckExist(p => p.Id != input.Id && p.LoginName == input.LoginName))
                throw new VFriendlyException("登录名已经存在");

            var entity = repo.GetEntityOrDefault(input.Id) ?? throw new VFriendlyException("找不到用户");

            input.Password = input.Password.IsNullOrEmpty()
                ? entity.Password
                : new PasswordHasher<User>().HashPassword(entity, input.Password);

            input.MapTo(entity);
            repo.SaveChanges();

            _authCache.ClearUser(VSession.TenantId, entity.Id);
        }

        public void Delete(DeleteInput input)
        {
            var repo = GetRepository(VEfDataFilters.Passive);

            var entity = repo.GetEntityOrDefault(input.Id) ?? throw new VFriendlyException("找不到用户");
            if (entity.IsStaticUser) throw new VFriendlyException("不能删除内置用户");
            
            entity.LoginName += "_deleted_" + entity.Id;
            repo.Delete(entity);
            repo.SaveChanges();

            _authCache.ClearUser(VSession.TenantId, entity.Id);
        }
    }
}
