﻿using System;
using System.Linq;
using Unity;
using VCommon;
using VCommon.Ioc;
using VCommon.VApplication.DataStore;
using VCommon.VAutoMapper;
using VCommon.VOpenApi.VAspNetCore;
using VCommonCoreExample.Caching;
using VCommonCoreExample.DataStore.Models;
using VCommonCoreExample.Entity;
using VCommonCoreExample.EntityFrameworkCore;

namespace VCommonCoreExample.DataStore
{
    public class UserStore : IUserStore, ISingletonIocClass
    {
        private const int NormalSessionExpireDays = 1;
        private const int RememberMeSessionExpireDays = 7;

        private readonly SessionCache _sessionCache;
        private readonly AuthCache _authCache;

        [Dependency]
        public IIocManager IocManager { get; set; }

        private ExampleDbContext GetDbContext() => IocManager.Resolve<ExampleDbContext>();

        public UserStore(SessionCache sessionCache, AuthCache authCache)
        {
            _sessionCache = sessionCache;
            _authCache = authCache;
        }

        public UserLoginModel GetLoginInfo(string tenantCode, string loginName, out string message)
        {
            using var db = GetDbContext();

            if (tenantCode.IsNullOrWhiteSpace())
            {
                message = null;
                return db.Users.Where(p => p.TenantId == null && p.IsAbolish == false && p.LoginName == loginName)
                    .ProjectionToArray<User, UserLoginModel>()
                    .FirstOrDefault();
            }

            var tid = db.Tenants
                .Where(p => p.IsAbolish == false && p.IsEnable && p.Code == tenantCode)
                .Select(p => (Guid?)p.Id).FirstOrDefault();

            if (tid.HasValue == false)
            {
                message = "租户代码无效或租户未启用";
                return null;
            }

            message = null;
            return db.Users.Where(p => p.TenantId == tid && p.IsAbolish == false && p.LoginName == loginName)
                .ProjectionToArray<User, UserLoginModel>()
                .FirstOrDefault();
        }

        public void ValidateToken(string token, out Guid? tenantId, out Guid userId)
        {
            var scm = _sessionCache.Get(token);
            if (scm == null) throw new VFriendlyException("会话已失效");
            var ks = _sessionCache.GetKickSign(scm.TenantId, scm.UserId);
            if (scm.KickSign != ks)
            {
                _sessionCache.Clear(token);
                throw new VFriendlyException("会话已失效：用户已重新登录");
            }

            //Check exist and tenant/user delete/disable/passChanged

            if (scm.TenantId.HasValue && true != _authCache.CheckTenantEnable(scm.TenantId.Value))
            {
                _sessionCache.Clear(token);
                throw new VFriendlyException("会话已失效：租户不存在或未启用");
            }

            var ucm = _authCache.GetUser(scm.TenantId, scm.UserId);
            if (null == ucm)
            {
                _sessionCache.Clear(token);
                throw new VFriendlyException("会话已失效：用户不存在");
            }

            if (false == ucm.IsEnable)
            {
                _sessionCache.Clear(token);
                throw new VFriendlyException("会话已失效：用户未启用");
            }

            if (scm.HashedPassword != ucm.Password)
            {
                _sessionCache.Clear(token);
                throw new VFriendlyException("会话已失效：密码已变更");
            }

            // extend expire
            _sessionCache.ExtendExpire(token, scm.TenantId, scm.UserId, scm.Remember ? RememberMeSessionExpireDays : NormalSessionExpireDays);

            tenantId = scm.TenantId;
            userId = scm.UserId;
        }
    }
}
